==> cvstrac-1.1.0.tar.gz: cvstrac-1.1.0/common.c <==
/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Routines shared by many pages
*/
#include "config.h"
#include "common.h"

/*
** Output a string with the following substitutions:
**
**     %T      The title of the current page.
**     %N      Project name
**     %V      CVSTrac version number
**     %%      The character '%'
*/
static void output_with_subst(const char *zText, const char *zTitle){
  int i;
  while( zText[0] ){
    for(i=0; zText[i] && zText[i]!='%'; i++){}
    if( i>0 ) cgi_append_content(zText, i);
    if( zText[i]==0 ) break;
    switch( zText[i+1] ){
      case 'T':
        zText += i+2;
        cgi_printf("%h", zTitle);
        break;
      case 'N':
        zText += i+2;
        cgi_printf("%h", g.zName);
        break;
      case 'V':
        zText += i+2;
        cgi_printf("%h", "@VERSION@");
        break;
      default:
        zText += i+1;
        break;
    }
  }
}

/*
** Read the whole contents of a file into memory obtained from
** malloc().  Return a pointer to the file contents.  Be sure
** the string is null terminated.
**
** A NULL pointer is returned if the file could not be read
** for any reason.
*/
static char *common_readfile(const char *zFilename) {
  FILE *fp;
  char *zContent = NULL;
  size_t n;

  if ((fp = fopen(zFilename, "r")) != NULL) {
    fseek(fp, 0, SEEK_END);
    if ((n = ftell(fp)) > 0) {
      if ((zContent = (char *)malloc(n+1)) == NULL) {
        fclose(fp);
        return NULL;
      }
      fseek(fp, 0, SEEK_SET);
      if ((n = fread(zContent, 1, n, fp)) == 0) {
        free(zContent);
        fclose(fp);
        return NULL;
      }
      zContent[n] = '\0';
    }
    else {
      zContent = strdup("");
    }
    fclose(fp);
  }
  return zContent;
}


/*
** Generate an HTML header common to all web pages.  zTitle is the
** title for the page.  azLinks is an array of URI/Name pairs that
** are used to generate quick-links on the title bar.
*/
void common_header(const char *zTitle, const char **azLink, ...){
  int i = 0;
  int brk;
  char *zHeader = 0;
  va_list ap;
  char *zTitleTxt;

  va_start(ap, azLink);
  zTitleTxt = vmprintf(zTitle, ap);
  va_end(ap);
  zHeader = db_short_query("SELECT value FROM config WHERE name='header'");
  if( zHeader && zHeader[0] ){
    char *z;
    if( zHeader[0]=='/' && (z = common_readfile(zHeader))!=0 ){
      free(zHeader);
      zHeader = z;
    }
    output_with_subst(zHeader, zTitleTxt);
    free(zHeader);
  }else{
    @ <html>
    @ <head>
    @ <title>%h(g.zName): %h(zTitleTxt)</title>
    @ </head>
    @ <body bgcolor="white">
  }
  @ <table width="100%%" cellpadding=2 border=0>
  @ <tr><td bgcolor="%s(BORDER1)" class="border1">
  @ <table width="100%%" border=0 cellpadding=2 cellspacing=0>
  @ <tr bgcolor="%s(BG1)" class="bkgnd1">
  @ <td valign="top" align="left">
  @ <big><b>%h(g.zName) - %h(zTitleTxt)</b></big><br>
  if( !g.isAnon ){
    @ <small>Logged in as <i>%h(g.zUser)</i></small>
  }else{
    @ <small>Not logged in</small>
  }
  @ </td>
  @ <td valign="bottom" align="right">
  if( azLink ){
    for(i=0; azLink[i*2]; i++){}
    brk = i<5 ? i+1 : i/2;
    brk = (brk-1)*2;
    for(i=0; azLink[i] && azLink[i+1]; i+=2){
      @ [<a href="%h(azLink[i])"><nobr>%h(azLink[i+1])</nobr></a>]&nbsp;
      if( i==brk ){
        @ <br>
      }
    }
  }
  if( i==0 ){
    @ &nbsp;
  }
  @ </td></tr></table>
  @ </td></tr></table>
  free(zTitleTxt);
}

/*
** Generate a common footer
*/
void common_footer(void){
  char *zFooter;
  zFooter = db_short_query("SELECT value FROM config WHERE name='footer'");
  if( zFooter && zFooter[0] ){
    char *z;
    if( zFooter[0]=='/' && (z = common_readfile(zFooter))!=0 ){
      free(zFooter);
      zFooter = z;
    }
    output_with_subst(zFooter, "");
    free(zFooter);
  }else{
    @ <p><small><small>
    @ <a href="about">CVSTrac version @VERSION@</a>
    @ </small></small></p>
    @ </body></html>
  }
}

/*
** Generate an about screen
**
** WEBPAGE: /about
*/
void common_about(void){
  static const char *azLink[] = { 
     "index", "Home", 
     0
  };
  login_check_credentials();
  common_header("About This Server", azLink);
  @ <p>This website is implemented using CVSTrac version @VERSION@.</p>
  @
  @ <p>CVSTrac implements a low-ceremony bug tracking system for small
  @ to medium-sized projects using CVS for version control.  For additional
  @ information, visit the CVSTrac homepage at</p>
  @ <blockquote>
  @ <a href="http://www.hwaci.com/sw/cvstrac/">
  @ http://www.hwaci.com/sw/cvstrac/
  @ </a>
  @ </blockquote>
  @
  @ <p>&copy; 2002 by <a href="mailto:drh@hwaci.com">D. Richard Hipp</a>.
  @ The CVSTrac server is released under the terms of the
  @ <a href="http://www.gnu.org/copyleft/gpl.html">GNU Public License</a>.</p>
  common_footer(); 
}

==> cvstrac-1.1.1.tar.gz: cvstrac-1.1.1/common.c <==
/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Routines shared by many pages
*/
#include "config.h"
#include "common.h"

/*
** Output a string with the following substitutions:
**
**     %T      The title of the current page.
**     %N      Project name
**     %V      CVSTrac version number
**     %%      The character '%'
*/
static void output_with_subst(const char *zText, const char *zTitle){
  int i;
  while( zText[0] ){
    for(i=0; zText[i] && zText[i]!='%'; i++){}
    if( i>0 ) cgi_append_content(zText, i);
    if( zText[i]==0 ) break;
    switch( zText[i+1] ){
      case 'T':
        zText += i+2;
        cgi_printf("%h", zTitle);
        break;
      case 'N':
        zText += i+2;
        cgi_printf("%h", g.zName);
        break;
      case 'V':
        zText += i+2;
        cgi_printf("%h", "@VERSION@");
        break;
      case '%':
        zText += i+2;
        cgi_printf("%%");
        break;
      default:
        zText += i+1;
        cgi_printf("%%");
        break;
    }
  }
}

/*
** Return a compressed version of a message which has multiple
** whitespaces and newlines reduces to single whitespace and newline.
*/
char *output_compress_message(const char *msg)
{
  char *z;
  char *zIn;
  char *zOut;

  if( !msg )
    return 0;
  z = strdup(msg);
  if( !z )
    return 0;
  zOut = zIn = z;
  while( *zIn ){
    if( *zIn == ' ' || *zIn == '\t' ){
      *zOut++ = ' ';
      while( *zIn == ' ' || *zIn == '\t' )
        zIn++;
    }
    else if( *zIn == '\r' || *zIn == '\n' ){
      *zOut++ = '\n';
      while( *zIn == '\r' || *zIn == '\n' )
        zIn++;
    }
    else
      *zOut++ = *zIn++;
  }
  *zOut = '\0';
  return z;
}

/*
** Read the whole contents of a file into memory obtained from
** malloc().  Return a pointer to the file contents.  Be sure
** the string is null terminated.
**
** A NULL pointer is returned if the file could not be read
** for any reason.
*/
static char *common_readfile(const char *zFilename) {
  FILE *fp;
  char *zContent = NULL;
  size_t n;

  if ((fp = fopen(zFilename, "r")) != NULL) {
    fseek(fp, 0, SEEK_END);
    if ((n = ftell(fp)) > 0) {
      if ((zContent = (char *)malloc(n+1)) == NULL) {
        fclose(fp);
        return NULL;
      }
      fseek(fp, 0, SEEK_SET);
      if ((n = fread(zContent, 1, n, fp)) == 0) {
        free(zContent);
        fclose(fp);
        return NULL;
      }
      zContent[n] = '\0';
    }
    else {
      zContent = strdup("");
    }
    fclose(fp);
  }
  return zContent;
}


/*
** Generate an HTML header common to all web pages.  zTitle is the
** title for the page.  azLinks is an array of URI/Name pairs that
** are used to generate quick-links on the title bar.
*/
void common_header(const char *zTitle, const char **azLink, ...){
  int i = 0;
  int brk;
  char *zHeader = 0;
  va_list ap;
  char *zTitleTxt;

  va_start(ap, azLink);
  zTitleTxt = vmprintf(zTitle, ap);
  va_end(ap);
  zHeader = db_short_query("SELECT value FROM config WHERE name='header'");
  if( zHeader && zHeader[0] ){
    char *z;
    if( zHeader[0]=='/' && (z = common_readfile(zHeader))!=0 ){
      free(zHeader);
      zHeader = z;
    }
    output_with_subst(zHeader, zTitleTxt);
    free(zHeader);
  }else{
    @ <html>
    @ <head>
    @ <title>%h(g.zName): %h(zTitleTxt)</title>
    @ </head>
    @ <body bgcolor="white">
  }
  @ <table width="100%%" cellpadding=2 border=0>
  @ <tr><td bgcolor="%s(BORDER1)" class="border1">
  @ <table width="100%%" border=0 cellpadding=2 cellspacing=0>
  @ <tr bgcolor="%s(BG1)" class="bkgnd1">
  @ <td valign="top" align="left">
  @ <big><b>%h(g.zName) - %h(zTitleTxt)</b></big><br>
  if( !g.isAnon ){
    @ <small>Logged in as <i>%h(g.zUser)</i></small>
  }else{
    @ <small>Not logged in</small>
  }
  @ </td>
  @ <td valign="bottom" align="right">
  if( azLink ){
    for(i=0; azLink[i*2]; i++){}
    brk = i<5 ? i+1 : i/2;
    brk = (brk-1)*2;
    for(i=0; azLink[i] && azLink[i+1]; i+=2){
      @ [<a href="%h(azLink[i])"><nobr>%h(azLink[i+1])</nobr></a>]&nbsp;
      if( i==brk ){
        @ <br>
      }
    }
  }
  if( i==0 ){
    @ &nbsp;
  }
  @ </td></tr></table>
  @ </td></tr></table>
  @ <p>
  free(zTitleTxt);
}

/*
** Generate a common footer
*/
void common_footer(void){
  char *zFooter;
  zFooter = db_short_query("SELECT value FROM config WHERE name='footer'");
  if( zFooter && zFooter[0] ){
    char *z;
    if( zFooter[0]=='/' && (z = common_readfile(zFooter))!=0 ){
      free(zFooter);
      zFooter = z;
    }
    output_with_subst(zFooter, "");
    free(zFooter);
  }else{
    @ <p><small><small>
    @ <a href="about">CVSTrac version @VERSION@</a>
    @ </small></small></p>
    @ </body></html>
  }
}

/*
** Generate an about screen
**
** WEBPAGE: /about
*/
void common_about(void){
  static const char *azLink[] = { 
     "index", "Home", 
     0
  };
  login_check_credentials();
  common_header("About This Server", azLink);
  @ <p>This website is implemented using CVSTrac version @VERSION@.</p>
  @
  @ <p>CVSTrac implements a low-ceremony bug tracking system for small
  @ to medium-sized projects using CVS for version control.  For additional
  @ information, visit the CVSTrac homepage at</p>
  @ <blockquote>
  @ <a href="http://www.cvstrac.org/">
  @ http://www.cvstrac.org/
  @ </a>
  @ </blockquote>
  @
  @ <p>Copyright &copy; 2002 by <a href="mailto:drh@hwaci.com">D. Richard Hipp</a>.
  @ The CVSTrac server is released under the terms of the
  @ <a href="http://www.gnu.org/copyleft/gpl.html">GNU Public License</a>.</p>
  common_footer(); 
}
